'''This code is licenced under a Creative Commons
Attribution-NonCommercial-ShareAlike 3.0 Unported Licence.
Created by Phil Wagner 2011 www.brokenairplane.com. This code allows data logging and uploading to Google Spreadsheets.
It should remain free and be an opportunity for students to do Science where
the cost of Science Equipment might be to expensive. Additional features can be added
but this paragraph should always remain in the code.'''

#Future features - Additional Sensors, Review, Display Data/Graph on Phone, Store larger data on SD Card or connect via serial
#interface with Arduino/ADK

#Helpful Resources:
#http://code.google.com/p/android-scripting/ (sl4a)
#http://code.google.com/ (Google APIs and resources)
#http://www.adaptcode.com/using-google-spreadsheets-as-a-database-in-the-cloud/
#http://gdata-python-client.googlecode.com/svn/trunk/pydocs/gdata.spreadsheet.text_db.html

import android
import time
import gdata.spreadsheet.service
import gdata.spreadsheet.text_db
import string

droid = android.Android()

dt=100 #Time between sensing

def countdown():
    #3 Second Countdown and Vibrate
    for i in range(1,4):
        droid.vibrate(100)
        time.sleep(1)
    droid.vibrate(400)
    return

def makeDialog(title='',postext='',negtext=''):
    droid.dialogCreateAlert(title)
    droid.dialogSetPositiveButtonText(postext)
    if negtext != '':
        droid.dialogSetNegativeButtonText(negtext)
    droid.dialogShow()
    return droid.dialogGetResponse().result

def login(myEmail='example@gmail.com', password=''):    #Get Email and Password and login
    while password == '':
        email = droid.dialogGetInput("Email", "Enter the email account where the Data will be uploaded to:",myEmail).result
        password=droid.dialogGetPassword("Password", "Enter the account password where the Data will be uploaded to:").result
        droid.dialogCreateSpinnerProgress("Logging into Gmail" ,"Please Wait")
        droid.dialogShow()
        try:
            client = gdata.spreadsheet.text_db.DatabaseClient()
            client.SetCredentials(username=email, password=password)
        except gdata.spreadsheet.text_db.BadCredentials:
            droid.dialogDismiss()
            redo = makeDialog("Username or password incorrect.","OK")
            password =''
            continue
        else:
            droid.dialogDismiss()
            return client

def spreadSheetSetup(client, defaultTitle = 'Science Data'):
    client.ssl = True
    versionNumber = client.GetDatabases(name=defaultTitle)
    if len(versionNumber) > 0:
        understood = makeDialog(('Science Data %d' % len(versionNumber)), "OK")
        spreadsheetTitle = defaultTitle+' '+str(len(versionNumber))
    else:
        spreadsheetTitle = defaultTitle
    return client.CreateDatabase(spreadsheetTitle)

def time2Sense(defaultTime=5000):
    maxTime =0
    #Enter time for sensing
    while maxTime == 0: ##How long to sense?
        try:
            maxTime=int(droid.dialogGetInput("Time", "How long to collect data (in milliseconds):",str(defaultTime)).result)
        except TypeError:
            makeDialog("Please enter a numeric time", "OK")
            droid.dialogSetPositiveButtonText("OK")
            continue
        if maxTime > 30000: #30 seconds max for optimal upload
            check = makeDialog("Warning this could take a long time to upload.","I accept the risks","New time")
            if check['which'] == 'positive':
                return maxTime
            else:
                maxTime = 0
                continue
        return maxTime

def senseAccel(endTime,dt=100):    #Sense and store accel data
    timeSensed=0
    tempScience = []
    droid.startSensingTimed(2,dt) #dt = time between sensings
    droid.dialogCreateHorizontalProgress("Sensing" ,"Time Remaining",endTime)
    droid.dialogShow()
    while timeSensed <= endTime:
        tempScience.append(droid.sensorsReadAccelerometer().result) #add new accel data to list
        time.sleep(dt/1000.0)
        timeSensed+=dt
        droid.dialogSetCurrentProgress(timeSensed)
    droid.vibrate(300)
    droid.stopSensing()
    return tempScience

def main():
    db =spreadSheetSetup(login())
    endTime = time2Sense()
    ready = makeDialog("Are you ready", "Let's do science!", "Cancel")
    if ready['which'] == 'positive':
        countdown()
        science = senseAccel(endTime,dt)
        table=db.CreateTable('Accelerometer',['t','x','y','z']) #Beware spreadsheet column names cannot be uppercase if you are going to add to them later http://code.google.com/p/gdata-python-client/issues/detail?id=363
        droid.dialogCreateHorizontalProgress('Progress','Uploading to Docs',len(science)*3 )
        droid.dialogShow()
        progress = 0
        time = 0
        for data in science:
            table.AddRecord({'t':str(time),'x':str(data[0]),'y':str(data[1]),'z':str(data[2])}) #AddRecord requires strings as arguements str()
            progress+=3
            time+=dt
            droid.dialogSetCurrentProgress(progress)
        makeDialog("Go check your data on Docs","OK")

if __name__ == "__main__":
        main()
